use rppal::gpio::{Gpio, InputPin, Level};
use spaceapi_dezentrale_client::ClientBuilder;
use std::{env, time::Duration};
use tokio::time::sleep;

// Static delays
static RECHECK_DELAY: Duration = Duration::from_secs(5);
static ANTI_BOUNCE_DELAY: Duration = Duration::from_secs(1);

#[derive(Debug)]
enum DoorStatus {
    Open,
    Closed,
}

#[tokio::main]
async fn main() {
    env_logger::init();

    let door_pin = env::var("DOOR_PIN").unwrap_or("27".to_string());
    let door_pin = door_pin.parse().expect("A number for the DOOR_PIN");

    // setup
    log::debug!("Setup GPIO");
    let gpio = Gpio::new().unwrap();
    let pin = gpio.get(door_pin).unwrap().into_input();

    log::debug!("Build client");
    let spaceapi_client = ClientBuilder::new()
        .api_key(&env::var("API_KEY").expect("Set API_KEY in environment"))
        .base_url(&env::var("SPACEAPI_URL").expect("Set SPACEAPI_URL in environment"))
        .build()
        .unwrap();

    loop {
        log::debug!("Checking door status");
        let push_result = match check_door(&pin).await {
            DoorStatus::Open => {
                log::debug!("Door is open, sending keep open to server");
                spaceapi_client.keep_open().await.map(|_| ())
            }
            DoorStatus::Closed => {
                log::debug!("Door is closed, sending explicit close to server");
                spaceapi_client.close().await
            }
        };
        let _ = push_result.map_err(|err| format!("Problem while pushing door status: {err}"));

        sleep(RECHECK_DELAY).await;
        log::debug!("Waiting {RECHECK_DELAY:?} secs to read the door status");
    }
}

async fn check_door(pin: &InputPin) -> DoorStatus {
    // reading space status from door
    let mut pin_state = pin.read();

    // Anti bounce verification
    loop {
        sleep(ANTI_BOUNCE_DELAY).await;
        let pin_state_check = pin.read();
        if pin_state_check == pin_state {
            break;
        } else {
            pin_state = pin_state_check
        }
    }

    log::debug!("Pin state is {pin_state}");
    if pin_state == Level::Low {
        DoorStatus::Open
    } else {
        DoorStatus::Closed
    }
}
